using System.Text.Json;
using System.Text.Json.Serialization;

namespace Gatus.To.MonA.Models;

public record AppSettings(GatusConfiguration GatusConfiguration,
                          GatusToMonaConfiguration GatusToMonaConfiguration,
                          MonAConfiguration MonAConfiguration) {

    public static AppSettings Get(IConfigurationRoot configurationRoot) {
        AppSettings result;
#pragma warning disable IL3050, IL2026
        GatusConfiguration? gatusConfiguration = configurationRoot.GetSection(nameof(GatusConfiguration)).Get<GatusConfiguration>();
        GatusToMonaConfiguration? gatusToMonaConfiguration = configurationRoot.GetSection(nameof(GatusToMonaConfiguration)).Get<GatusToMonaConfiguration>();
        MonAConfiguration? monAConfiguration = configurationRoot.GetSection(nameof(MonAConfiguration)).Get<MonAConfiguration>();
#pragma warning restore IL3050, IL2026
        if (gatusConfiguration is null
            || gatusToMonaConfiguration is null
            || monAConfiguration is null
            || gatusToMonaConfiguration?.Company is null) {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers) {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
        result = new(gatusConfiguration,
                     gatusToMonaConfiguration,
                     monAConfiguration);
        Verify(result);
        return result;
    }

    private static void Verify(AppSettings appSettings) {
        if (string.IsNullOrEmpty(appSettings.GatusToMonaConfiguration.Company))
            throw new Exception("Company name must have a value!");
        if (appSettings.MonAConfiguration.Minutes < 1)
            throw new Exception("MonA Minutes must have a value more than zero!");
        if (appSettings.MonAConfiguration.Expected < 1)
            throw new Exception("MonA Expected must have a value more than zero!");
        TimeSpan timeSpan = new(DateTime.MinValue.AddMinutes(appSettings.MonAConfiguration.Minutes).Ticks - DateTime.MinValue.Ticks);
        if (appSettings.GatusToMonaConfiguration.MillisecondsDelay < timeSpan.TotalMilliseconds)
            throw new Exception($"MonA doesn't support more than once per five minutes ({timeSpan.TotalMilliseconds})!");
    }

    public override string ToString() {
        string result = JsonSerializer.Serialize(this, AppSettingsSourceGenerationContext.Default.AppSettings);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(AppSettings))]
internal partial class AppSettingsSourceGenerationContext : JsonSerializerContext {
}